/*
 * Written by Stephen henson (steve@openssl.org) for the OpenSSL project
 * 2011.
 */
/* ====================================================================
 * Copyright (c) 2011 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include "cryptlib.h"
#ifdef OPENSSL_FIPS
# include <openssl/fips.h>
# include <openssl/fips_rand.h>
# include <openssl/rand.h>
#endif
#ifdef NETAPP
#include <openssl/conf.h>
#include <openssl/evp.h>
#include <openssl/err.h>

/* Perform any essential FIPS initialization operations.
 * Currently reads and applies configuration
 *
 * NOTE: This routine is invoked currently from these APIs:
 * EVP_add_cipher(), EVP_add_digest(), FIPS_mode() and
 * FIPS_mode_set()
 *
 * Adding it in EVP_add_cipher/digest ensures the library
 * initialization routines like OpenSSL_add_all_algorithms()
 * and SSL_library_init() are covered. Adding it in FIPS_mode*
 * APIs ensures OpenSSL to OpenSSL-FIPS transition happens
 * only after initialization of the FIPS mode as per the
 * configuration. Although, transition from OpenSSL library
 * calls to FIPS-canister contained CRYPTO calls should happen
 * after checking of FIPS_mode(), we may have to revisit this
 * if the OpenSSL design is changed to do such transition
 * without of explicit check of FIPS_mode() as such.
 *
 */
void FIPS_init(void)
{
    if (get_OPENSSL_init_done()) {
        CONF_MODULE *md = NULL;
        ALG_MODULE_DATA *alg_data = NULL;
        CRYPTO_THREADID cur;

        CRYPTO_THREADID_current(&cur);

        // So the obvious optimization is that we need to avoid doing the
        // significant optimization work every time through here. Unfortunately,
        // we can't just assume once is enough since the config could be
        // unloaded without unloading the whole library. So, we have to check
        // the alg_data thing to convince ourselves a config really is there.
        // Even more unfortunately, the CONF_modules_load_file chain will end
        // up calling into here recursively, so we have to do some recursion
        // detection to avoid eventual deadlock there.

        CRYPTO_r_lock(CRYPTO_LOCK_ALG_CONF);
        if ((md = module_find("alg_section")) &&
            (alg_data = (ALG_MODULE_DATA *) CONF_module_get_usr_data(md))) {
            if ((alg_data->thr_id) &&
                (!CRYPTO_THREADID_cmp(alg_data->thr_id, &cur))) {
                CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);
                return;
            }
        }
        CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);

        CRYPTO_w_lock(CRYPTO_LOCK_FIPS_INIT);

        CRYPTO_r_lock(CRYPTO_LOCK_ALG_CONF);
        if ((md = module_find("alg_section")) &&
            (alg_data = (ALG_MODULE_DATA *) CONF_module_get_usr_data(md)) &&
            alg_data->init_done) {
            CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);
            CRYPTO_w_unlock(CRYPTO_LOCK_FIPS_INIT);
            return;
        }
        CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);

        EVP_add_alg_module();
        ERR_clear_error();
        CONF_modules_load_file(NULL /* default config file */,
                               NULL /* default section */,
                               CONF_MFLAGS_DEFAULT_SECTION |
                               CONF_MFLAGS_IGNORE_MISSING_FILE);

        CRYPTO_w_lock(CRYPTO_LOCK_ALG_CONF);
        if ((md = module_find("alg_section")) &&
            (alg_data = (ALG_MODULE_DATA *) CONF_module_get_usr_data(md))) {
            alg_data->init_done = 1;
        }
        CRYPTO_w_unlock(CRYPTO_LOCK_ALG_CONF);

        CRYPTO_w_unlock(CRYPTO_LOCK_FIPS_INIT);
    }
}
#endif  /* NETAPP */

int FIPS_mode(void)
{
    OPENSSL_init();
#ifdef NETAPP
    FIPS_init();
#endif /* NETAPP */
#ifdef OPENSSL_FIPS
    return FIPS_module_mode();
#else
    return 0;
#endif
}

int FIPS_mode_set(int r)
{
    OPENSSL_init();
#ifdef NETAPP
#ifdef OPENSSL_FIPS
    if ( r == 0 )
    {
        /* Check the configuration and disallow only if the
         * fips_mode=yes string is present in configuration
         */
        CONF_MODULE *md = NULL;
        ALG_MODULE_DATA *alg_data = NULL;

        CRYPTO_r_lock(CRYPTO_LOCK_ALG_CONF);
        if ((md = module_find("alg_section")) &&
            (alg_data = (ALG_MODULE_DATA *) CONF_module_get_usr_data(md)) &&
            (alg_data->fips_mode))
        {
            CRYPTOerr(CRYPTO_F_FIPS_MODE_SET, CRYPTO_R_FIPS_MODE_UNSET_NOT_ALLOWED);
            CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);
            return 0;
        }
        CRYPTO_r_unlock(CRYPTO_LOCK_ALG_CONF);
    }
    /*
     * See BURT 1111048
     *
     * This is a hack to fix an issue where we were creating encrypted RSA
     * keys in non-FIPS mode and then failing to be able to use them when
     * in FIPS mode.
     *
     * As a workaround, we turn FIPS mode off and then re-write the
     * keys after turning FIPS mode back on.
     *
     * This hack should not be abused...
     */
    if ( r == -1 )
    {
	r = 0;
    }
#endif
#endif /* NETAPP */
#ifdef OPENSSL_FIPS
# ifndef FIPS_AUTH_USER_PASS
#  define FIPS_AUTH_USER_PASS     "Default FIPS Crypto User Password"
# endif
    if (!FIPS_module_mode_set(r, FIPS_AUTH_USER_PASS))
        return 0;
    if (r)
        RAND_set_rand_method(FIPS_rand_get_method());
    else
        RAND_set_rand_method(NULL);
    return 1;
#else
    if (r == 0)
        return 1;
    CRYPTOerr(CRYPTO_F_FIPS_MODE_SET, CRYPTO_R_FIPS_MODE_NOT_SUPPORTED);
    return 0;
#endif
}
