/* o_init.c */
/*
 * Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
 * project.
 */
/* ====================================================================
 * Copyright (c) 2011 The OpenSSL Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit. (http://www.openssl.org/)"
 *
 * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    openssl-core@openssl.org.
 *
 * 5. Products derived from this software may not be called "OpenSSL"
 *    nor may "OpenSSL" appear in their names without prior written
 *    permission of the OpenSSL Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the OpenSSL Project
 *    for use in the OpenSSL Toolkit (http://www.openssl.org/)"
 *
 * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#include <e_os.h>
#include <openssl/err.h>
#ifdef OPENSSL_FIPS
# include <openssl/fips.h>
# include <openssl/rand.h>

# ifndef OPENSSL_NO_DEPRECATED
/* the prototype is missing in <openssl/fips.h> */
void FIPS_crypto_set_id_callback(unsigned long (*func)(void));
# endif
#endif
#ifdef NETAPP
static int OPENSSL_init_done = 0;

int get_OPENSSL_init_done(void)
        {
        return OPENSSL_init_done;
        }
#endif /* NETAPP */

/*
 * Perform any essential OpenSSL initialization operations. Currently only
 * sets FIPS callbacks
 */

void OPENSSL_init(void)
{
    static int done = 0;
    if (done)
        return;
    done = 1;
#ifdef OPENSSL_FIPS
    FIPS_set_locking_callbacks(CRYPTO_lock, CRYPTO_add_lock);
#ifdef NETAPP
    // It's amazing how deviously wrong the non-NetApp version of this is.
    //
    // If you look at CRYPTO_thread_id, it actually tries to call the stored
    // id_callback. So CRYPTO_set_id_callback(CRYPTO_thread_id) would actually
    // in infinite recursion, as would s/CRYPTO/FIPS_crypto/g. It's only
    // because there are distinct function pointers stored inside and outside
    // the FIPS module that it even works at all.
    //
    // But that's not even the worst part. The worst part is that
    // CRYPTO_thread_id itself is a terrible default for most platforms (and in
    // particular the ones we=NetApp care about): it falls back on the PID,
    // which will of course be the same for all threads in a given process. The
    // upshot of that is that any code inside the FIPS module that is comparing
    // CRYPTO_THREADID* which were set by CRYPTO_THREADID_current may come to
    // very mistaken conclusions about threads in the system. Fortunately, very
    // little code inside the current immutable FIPS module uses
    // CRYPTO_THREADID_cmp. Unfortunately, that which does relies on it to make
    // determinations about shared data (ie, whether or not a given thread
    // "owns" something or needs to lock). So that basically sets said code up
    // for pernicious race conditions. The two current victims are FIPS
    // initialization and the BN_BLINDING within a given RSA. FIPS
    // initialization ends up being protected by the higher-level
    // NetApp-specific initialization locks anyway. The race on the BN_BLINDING
    // ends up being incredibly difficult to cause corrution with (and thereby
    // detect), and results in resoundingly non-obvious symptoms when it does.
    FIPS_crypto_threadid_set_callback(CRYPTO_THREADID_get_callback());
#endif
# ifndef OPENSSL_NO_DEPRECATED
#ifdef NETAPP
    FIPS_crypto_set_id_callback(CRYPTO_get_id_callback());
#else
    FIPS_crypto_set_id_callback(CRYPTO_thread_id);
#endif
# endif
    FIPS_set_error_callbacks(ERR_put_error, ERR_add_error_vdata);
    FIPS_set_malloc_callbacks(CRYPTO_malloc, CRYPTO_free);
    RAND_init_fips();
#endif
#ifdef NETAPP
    OPENSSL_init_done = 1;
#endif /* NETAPP */
#if 0
    fprintf(stderr, "Called OPENSSL_init\n");
#endif
}
